@@ -1,37 +1,45 @@
+#![deny(missing_docs)]
+
 //! This crate provides a [Bevy](https://bevyengine.org/) plugin for integrating with
 //! the Steamworks SDK.
 //!
+//! ## Bevy Version Supported
+//!
+//! |Bevy Version |bevy\_steamworks|
+//! |:------------|:---------------|
+//! |git (main)   |git (develop)   |
+//! |0.6          |0.2, 0.3        |
+//! |0.5          |0.1             |
+//!
 //! ## Installation
 //! Add the following to your `Cargo.toml`:
 //!
 //! ```toml
 //! [dependencies]
-//! bevy-steamworks = "0.1.0"
+//! bevy-steamworks = "0.3"
 //! ```
 //!
-//! Ensure that your computer has all the needed
+//! Ensure that your build environment has all the needed
 //! [requirements](https://rust-lang.github.io/rust-bindgen/requirements.html) to use
 //! [bindgen](https://github.com/rust-lang/rust-bindgen).
 //!
 //! Download and install the [steamworks sdk](https://partner.steamgames.com/doc/sdk)
 //! and set the environment variable `STEAM_SDK_LOCATION` to point to it.
 //!
-//! At runtime, a "steam_appid.txt" file with the registered Steam App ID of the game
-//! is required in the same directory as the executable.
-//!
 //! ## Usage
 //!
-//! To add the plugin to your game, simply add the `SteamworksPlugin` to your
-//! `AppBuilder`.
+//! To add the plugin to your app, simply add the `SteamworksPlugin` to your
+//! `App`. This will require the `AppId` provided to you by Valve for initialization.
 //!
-//! ```rust
+//! ```rust no_run
 //! use bevy::prelude::*;
-//! use bevy_steamworks::SteamworksPlugin;
+//! use bevy_steamworks::*;
 //!
 //! fn main() {
-//!   App::build()
+//!   // Use the demo Steam AppId for SpaceWar
+//!   App::new()
 //!       .add_plugins(DefaultPlugins)
-//!       .add_plugin(SteamworksPlugin)
+//!       .add_plugin(SteamworksPlugin::new(AppId(480)))
 //!       .run()
 //! }
 //! ```
@@ -48,28 +56,30 @@
 //! returns an error, an error wil lbe logged (via `bevy_log`), but it will not
 //! panic. In this case, it may be necessary to use `Option<Res<Client>>` instead.
 //!
-//! ```rust
-//! use bevy_steamworks::{Client, FriendFlags};
+//! ```rust no_run
+//! use bevy::prelude::*;
+//! use bevy_steamworks::*;
 //!
 //! fn steam_system(steam_client: Res<Client>) {
-//!   for friend in client.friends().get_friends(FriendFlags::IMMEDIATE) {
+//!   for friend in steam_client.friends().get_friends(FriendFlags::IMMEDIATE) {
 //!     println!("Friend: {:?} - {}({:?})", friend.id(), friend.name(), friend.state());
 //!   }
 //! }
 //!
 //! fn main() {
-//!   App::build()
+//!   // Use the demo Steam AppId for SpaceWar
+//!   App::new()
 //!       .add_plugins(DefaultPlugins)
-//!       .add_plugin(SteamworksPlugin)
-//!       .add_startup_system(steam_system.system())
+//!       .add_plugin(SteamworksPlugin::new(AppId(480)))
+//!       .add_startup_system(steam_system)
 //!       .run()
 //! }
 //! ```
 
-use bevy_app::{AppBuilder, EventWriter, Plugin};
+use bevy_app::{App, EventWriter, Plugin};
 use bevy_ecs::{schedule::*, system::*};
-use bevy_log::error;
-use std::sync::{Arc, Mutex};
+use parking_lot::Mutex;
+use std::sync::Arc;
 pub use steamworks::*;
 
 struct SteamEvents<T> {
@@ -77,19 +87,31 @@ struct SteamEvents<T> {
     pending: Arc<Mutex<Vec<T>>>,
 }
 
-pub struct SteamworksPlugin;
+/// A Bevy [`Plugin`] for adding support for the Steam SDK.
+///
+/// [`Plugin`]: bevy_app::Plugin
+pub struct SteamworksPlugin(AppId);
+
+impl SteamworksPlugin {
+    /// Creates a new `SteamworksPlugin`. The provided `app_id` should correspond
+    /// to the Steam app ID provided by Valve.
+    pub fn new(app_id: impl Into<AppId>) -> Self {
+        Self(app_id.into())
+    }
+}
 
 impl Plugin for SteamworksPlugin {
-    fn build(&self, app: &mut AppBuilder) {
-        if app.world().contains_resource::<Client<ClientManager>>() {
+    fn build(&self, app: &mut App) {
+        if app.world.contains_resource::<Client<ClientManager>>() {
+            bevy_log::warn!("Attempted to add the Steamworks plugin multiple times!");
             return;
         }
-        match Client::init() {
-            Err(err) => error!("Failed to initialize Steamworks client: {}", err),
+        match Client::init_app(self.0) {
+            Err(err) => bevy_log::error!("Failed to initialize Steamworks client: {}", err),
             Ok((client, single)) => {
                 app.insert_resource(client.clone())
                     .insert_non_send_resource(single)
-                    .add_system(run_steam_callbacks.system().label(STEAM_CALLBACKS));
+                    .add_system(run_steam_callbacks.label(SteamworksSystem::RunCallbacks));
 
                 add_event::<AuthSessionTicketResponse>(app, &client);
                 add_event::<GameLobbyJoinRequested>(app, &client);
@@ -105,7 +127,19 @@ impl Plugin for SteamworksPlugin {
     }
 }
 
-const STEAM_CALLBACKS: &str = "run_steam_callbacks";
+/// A set of [`SystemLabel`]s for systems used by [`SteamworksPlugin`]
+///
+/// [`SystemLabel`]: bevy_ecs::system::SystemLabel
+#[derive(Debug, Clone, Copy, Eq, Hash, SystemLabel, PartialEq)]
+pub enum SteamworksSystem {
+    /// A system that runs the Steam SDK callbacks. Anything dependent on
+    /// Steam API results should run after this.
+    RunCallbacks,
+    /// A set of systems for flushing events from the Steam SDK into bevy.
+    /// If using [`EventReader`] with any of these events, it should be
+    /// scheduled after these systems.
+    FlushEvents,
+}
 
 fn run_steam_callbacks(client: NonSend<SingleClient>) {
     client.run_callbacks();
@@ -115,24 +149,25 @@ fn flush_events<T: Send + Sync + 'static>(
     events: ResMut<SteamEvents<T>>,
     mut output: EventWriter<T>,
 ) {
-    let mut pending = events.pending.lock().unwrap();
-    output.send_batch(pending.drain(0..));
+    let mut pending = events.pending.lock();
+    if !pending.is_empty() {
+        output.send_batch(pending.drain(0..));
+    }
 }
 
-fn add_event<T: Callback + Send + Sync + 'static>(
-    app: &mut AppBuilder,
-    client: &Client<ClientManager>,
-) {
+fn add_event<T: Callback + Send + Sync + 'static>(app: &mut App, client: &Client<ClientManager>) {
     let pending = Arc::new(Mutex::new(Vec::new()));
     let pending_in = pending.clone();
-    let callback = client.register_callback::<T, _>(move |evt| {
-        pending_in.lock().unwrap().push(evt);
-    });
-    let events = SteamEvents::<T> {
-        _callback: callback,
-        pending,
-    };
     app.add_event::<T>()
-        .insert_resource(events)
-        .add_system(flush_events::<T>.system().after(STEAM_CALLBACKS));
+        .insert_resource(SteamEvents::<T> {
+            _callback: client.register_callback::<T, _>(move |evt| {
+                pending_in.lock().push(evt);
+            }),
+            pending,
+        })
+        .add_system(
+            flush_events::<T>
+                .label(SteamworksSystem::FlushEvents)
+                .after(SteamworksSystem::RunCallbacks),
+        );
 }
